<?php

namespace Drupal\nextpre\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Url;
use Drupal\Core\Link;
use Drupal\Core\Block\BlockPluginInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\node\Entity\NodeType;

/**
 * Provides a 'Next Previous' block.
 *
 * @Block(
 *   id = "next_previous_block",
 *   admin_label = @Translation("Next Previous Block"),
 *   category = @Translation("Blocks")
 * )
 */
class NextPreviousBlock extends BlockBase implements BlockPluginInterface, ContainerFactoryPluginInterface {

  /**
   * The currently active request object.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * NextPreviousBlock class constructor.
   *
   * @param array $configuration
   *   Plugin configuration.
   * @param string $plugin_id
   *   The plugin id.
   * @param mixed $plugin_definition
   *   The plugin definition.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   Symfony request stack instance.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   An entity type manager instance.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    RequestStack $request_stack,
    EntityTypeManagerInterface $entity_type_manager
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->request = $request_stack->getCurrentRequest();
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('request_stack'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);

    $node_types = NodeType::loadMultiple();
    $node_type_titles = [];

    foreach ($node_types as $machine_name => $val) {
      $node_type_titles[$machine_name] = $val->label();
    }
    $form['content_type'] = [
      '#type' => 'radios',
      '#title' => $this->t('Content Types'),
      '#options' => $node_type_titles,
      '#default_value' => isset($this->configuration['content_type']) ? $this->configuration['content_type'] : '',
    ];
    $form['previous_text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Previous text'),
      '#description' => $this->t('Add your previous button name'),
      '#default_value' => isset($this->configuration['previous_text']) ? $this->configuration['previous_text'] : '',
    ];
    $form['next_text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Next text'),
      '#description' => $this->t('Add your next button name'),
      '#default_value' => isset($this->configuration['next_text']) ? $this->configuration['next_text'] : '',
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    parent::blockSubmit($form, $form_state);
    $values = $form_state->getValues();
    $this->configuration['content_type'] = $form_state->getValue('content_type');
    $this->configuration['previous_text'] = $values['previous_text'];
    $this->configuration['next_text'] = $values['next_text'];
    parent::blockSubmit($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $link = [];

    // Get the created time of the current node.
    $node = $this->request->attributes->get('node');
    if ($node) {
      $current_nid = $node->get('vid')->getValue()[0]['value'];
      $link['prev'] = $this->generatePrevious($current_nid);
      $link['next'] = $this->generateNext($current_nid);
    }
    return $link;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheMaxAge() {
    return 0;
  }

  /**
   * Lookup the previous node,youngest node which is still older than the node.
   *
   * @param string $current_nid
   *   Get current page node id.
   *
   * @return array
   *   A render array for a previous node.
   */
  private function generatePrevious($current_nid) {
    return $this->generateNextPrevious($current_nid, 'prev');
  }

  /**
   * Lookup the next node,oldest node which is still younger than the node.
   *
   * @param string $current_nid
   *   Get current page node id.
   *
   * @return array
   *   A render array for a next node.
   */
  private function generateNext($current_nid) {
    return $this->generateNextPrevious($current_nid, 'next');
  }

  /**
   * Lookup the next or previous node.
   *
   * @param string $current_nid
   *   Get current nid.
   * @param string $direction
   *   Get button name.
   *
   * @return array
   *   Find the alias of the next node.
   */
  private function generateNextPrevious($current_nid, $direction = 'next') {
    if ($direction === 'next') {
      $comparison_opperator = '>';
      $sort = 'ASC';
      $display_text = $this->configuration['next_text'];
      $class = "nextpre_next";
    }
    elseif ($direction === 'prev') {
      $comparison_opperator = '<';
      $sort = 'DESC';
      $display_text = $this->configuration['previous_text'];
      $class = "nextpre_previous";
    }

    // Lookup 1 node younger (or older) than the current node.
    $query = $this->entityTypeManager->getStorage('node')->getQuery();
    $next = $query->condition('vid', $current_nid, $comparison_opperator)
      ->condition('type', $this->configuration['content_type'])
      ->condition('status', 1)
      ->sort('vid', $sort)
      ->range(0, 1)
      ->execute();

    // If this is not the youngest (or oldest) node.
    if (!empty($next) && is_array($next)) {
      $next = array_values($next);
      $next = $next[0];

      // Find the alias of the next node.
      $nid = $next;
      $url = Url::fromRoute('entity.node.canonical', ['node' => $nid], []);
      $link = Link::fromTextAndUrl($display_text, Url::fromUri('internal:/' . $url->getInternalPath()));
      $link = $link->toRenderable();
      $link['#attributes'] = ['class' => ['nextpre__btn btn', $class]];
      return $link;
    }
  }

}
